using System;
using System.Collections.Generic;
using System.Configuration;
using System.Messaging;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Transactions;
using System.Text;

namespace Compensation
{
    public class Program
    {
        private const string serviceQueueName = @".\private$\MyService";
        private const string targetQueueName = @".\private$\MyTarget";
        private const string compensationQueueName = @".\private$\MyCompensator";

        static void Main(string[] args)
        {
            if (!(MessageQueue.Exists(serviceQueueName)))
            {
                MessageQueue.Create(serviceQueueName, true);
            }

            if (!(MessageQueue.Exists(targetQueueName)))
            {
                MessageQueue.Create(targetQueueName, true);
            }

            if (!(MessageQueue.Exists(compensationQueueName)))
            {
                MessageQueue.Create(compensationQueueName, true);
            }

            Console.WriteLine("Press any key to proceed.");
            Console.ReadKey(true);
            
            bool fail = (!(bool.Parse(ConfigurationManager.AppSettings["Succeed"])));

            try
            {
                using (TransactionScope transaction = new TransactionScope(TransactionScopeOption.Required))
                {
                    MyClient client = new MyClient("MyService");

                    client.Open();

                    client.FirstOperation("First set of data");

                    Console.WriteLine("Sent first message.");

                    client.SecondOperation("Second set of data");

                    Console.WriteLine("Sent second message.");

                    if (fail)
                    {
                        throw new Exception("Something bad.");
                    }

                    client.Close();

                    transaction.Complete();

                    Console.WriteLine("Messages sent.");
                }

            }
            catch (Exception)
            {
                Console.WriteLine("Caught exception.");
            }
            
            
            Console.ReadKey(true);
        }
    }
}
