#region Using directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Hosting;

#endregion

namespace WCFUnleashedSamples
{
    class Program
    {
        static void Main(string[] args)
        {
            using(WorkflowRuntime workflowRuntime = new WorkflowRuntime())
            {
                AutoResetEvent waitHandle = new AutoResetEvent(false);
                workflowRuntime.WorkflowCompleted += delegate(object sender, WorkflowCompletedEventArgs e) {waitHandle.Set();};
                workflowRuntime.WorkflowTerminated += delegate(object sender, WorkflowTerminatedEventArgs e)
                {
                    Console.WriteLine(e.Exception.Message);
                    waitHandle.Set();
                };


                Console.ForegroundColor = ConsoleColor.Yellow;
                Console.WriteLine("Basic Workflow Sample, press <ENTER>");
                Console.ReadLine();
                WorkflowInstance instance = workflowRuntime.CreateWorkflow(typeof(WCFUnleashedSamples.BasicWorkflow));
                instance.Start();
                waitHandle.WaitOne();


                
                Console.WriteLine("Custom Activities Workflow Sample, press <ENTER>");
                Console.ReadLine();
                instance = workflowRuntime.CreateWorkflow(typeof(WCFUnleashedSamples.CustomActivities));
                instance.Start();
                waitHandle.WaitOne();


                Console.WriteLine("Parameter Passing Sample, press <ENTER>");
                Console.ReadLine();
                Dictionary<string, object> param = new Dictionary<string, object>();
                param.Add("TextToPassIn", "A String To Print");
                instance = workflowRuntime.CreateWorkflow(typeof(WCFUnleashedSamples.ParameterPassing ), param);
                instance.Start();
                waitHandle.WaitOne();

                Console.WriteLine("Unsynched Parallel Sample, press <ENTER>");
                Console.ReadLine();
                instance = workflowRuntime.CreateWorkflow(typeof(WCFUnleashedSamples.UnsynchedParallel));
                instance.Start();
                waitHandle.WaitOne();

                Console.WriteLine("Synched Parallel Sample, press <ENTER>");
                Console.ReadLine();
                instance = workflowRuntime.CreateWorkflow(typeof(WCFUnleashedSamples.SynchedParallel));
                instance.Start();
                waitHandle.WaitOne();


                Console.WriteLine("Compensation Sample, press <ENTER>");
                Console.ReadLine();
                param = new Dictionary<string, object>();
                param.Add("IntegerInput", 10);
                instance = workflowRuntime.CreateWorkflow(typeof(WCFUnleashedSamples.CompensationWorkflow), param);
                instance.Start();
                waitHandle.WaitOne();
                

                Console.WriteLine("State Machine Sample, press <ENTER>");
                Console.ReadLine();
                instance = workflowRuntime.CreateWorkflow(typeof(WCFUnleashedSamples.TimeDelayStateMachine));
                instance.Start();
                waitHandle.WaitOne();


                Console.ForegroundColor = ConsoleColor.Green;
                Console.WriteLine("Samples Complete, press <ENTER> to exit");
                Console.ReadLine();
            }
        }
    }
}
