﻿using System;

using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Xml;


namespace SimpleClient
{
    class Program
    {
        static void Main(string[] args)
        {
            DemoOne();
            DemoTwo();
            DemoThree();
        }

        // re-using the same client, preserves context
        private static void DemoOne()
        {
            Console.WriteLine("Press Enter to Send a Message and reuse proxy");
            Console.ReadLine();
            IWorkflowClient iwc = new IWorkflowClient(new NetTcpContextBinding(),
                new EndpointAddress("net.tcp://localhost:10001/Intro1"));

            IContextManager icm = iwc.InnerChannel.GetProperty<IContextManager>();

            if (null != icm) Console.WriteLine("Context contains {0} elements", icm.GetContext().Count);




            string s = iwc.Hello("message1");
            Console.WriteLine("the service returned the message '{0}'", s);
            CheckAndPrintContext(icm);
            s = iwc.Hello("message2");
            icm = iwc.InnerChannel.GetProperty<IContextManager>();
            CheckAndPrintContext(icm);
            Console.WriteLine("the service returned the message '{0}'", s);
            
            Console.WriteLine("Press Enter to Continue");
            Console.ReadLine();
        }

        // show this not working using a second client
        private static void DemoTwo()
        {
            Console.WriteLine("Press Enter to Send a Message (it will break this time)");
            Console.ReadLine();
            IWorkflowClient iwc = new IWorkflowClient(new NetTcpContextBinding(),
                new EndpointAddress("net.tcp://localhost:10001/Intro1"));

            IContextManager icm = iwc.InnerChannel.GetProperty<IContextManager>();

            if (null != icm) Console.WriteLine("Context contains {0} elements", icm.GetContext().Count);




            string s = iwc.Hello("message1");
            Console.WriteLine("the service returned the message '{0}'", s);

            CheckAndPrintContext(icm);
            iwc = new IWorkflowClient(new NetTcpContextBinding(),
               new EndpointAddress("net.tcp://localhost:10001/Intro1"));

            s = iwc.Hello("message2");
            Console.WriteLine("the service returned the message '{0}'", s);
            
            icm = iwc.InnerChannel.GetProperty<IContextManager>();
            CheckAndPrintContext(icm);
            Console.WriteLine("Press Enter to Continue");
            Console.ReadLine();
        }

        // show this working with a second client by caching the context
        private static void DemoThree()
        {
            Console.WriteLine("Press Enter to Send a Message (we'll cache the context and apply it to the new proxy)");
            Console.ReadLine();
            IWorkflowClient iwc = new IWorkflowClient(new NetTcpContextBinding(),
                new EndpointAddress("net.tcp://localhost:10001/Intro1"));

            IContextManager icm = iwc.InnerChannel.GetProperty<IContextManager>();

            if (null != icm) Console.WriteLine("Context contains {0} elements", icm.GetContext().Count);




            string s = iwc.Hello("message1");
            Console.WriteLine("the service returned the message '{0}'", s);

            CheckAndPrintContext(icm);
          
            IDictionary<string, string> context = icm.GetContext();
            icm = null;
            iwc = new IWorkflowClient(new NetTcpContextBinding(),
               new EndpointAddress("net.tcp://localhost:10001/Intro1"));
            icm = iwc.InnerChannel.GetProperty<IContextManager>();
            icm.SetContext(context);
            s = iwc.Hello("message2");
            Console.WriteLine("the service returned the message '{0}'", s);

            icm = iwc.InnerChannel.GetProperty<IContextManager>();
            CheckAndPrintContext(icm);
            Console.WriteLine("Press Enter to Exit");
            Console.ReadLine();
        }
        
        
        private static void CheckAndPrintContext(IContextManager icm)
        {
            if (null != icm) Console.WriteLine("Context contains {0} elements", icm.GetContext().Count);
            if (null != icm)
            {
                if (icm.GetContext().Count > 0)
                {
                    // Console.WriteLine("Here is the context: {0}", icm.GetContext().Keys[0]);
                    foreach (string xmlName in icm.GetContext().Keys)
                    {
                        Console.WriteLine("key : {0}", xmlName);
                        Console.WriteLine("value : {0}", icm.GetContext()[xmlName]);
                    }
                }
            }
        }

    }


    class IWorkflowClient : ClientBase<Intro1.IWorkflow1>, Intro1.IWorkflow1
    {


        public IWorkflowClient() : base() { }
        public IWorkflowClient(Binding binding, EndpointAddress address) : base(binding, address) { }
        #region IWorkflow1 Members

        public string Hello(string message)
        {
            return base.Channel.Hello(message);
        }

        #endregion
    }
}
