using System;
using System.IdentityModel.Selectors;

namespace Client
{
    public class Program
    {
        public static void Main(string[] args)
        {
            Console.WriteLine("Press any key when the service is ready.");
            Console.ReadKey(true);

            try
            {
                decimal result = 0;
                using (DerivativesCalculatorClient proxy =
                    new DerivativesCalculatorClient())
                {
                    result = proxy.CalculateDerivative(
                        new string[] { "MSFT" },
                        new decimal[] { 3 },
                        new string[] { });
                    proxy.Close();
                }
                Console.WriteLine(string.Format("Result: {0}", result));
            }
            catch (System.IdentityModel.Selectors.CardSpaceException cse)
            {
                Console.WriteLine("Generic Cardspace exception:" + cse.Message);
            }
            catch (IdentityValidationException)
            {
                Console.WriteLine("Recipient's certificate was not valid");
            }
            catch (System.IdentityModel.Selectors.UserCancellationException)
            {
                Console.WriteLine("User cancelled");
            }
            catch (UntrustedRecipientException)
            {
                Console.WriteLine("User does not trust the recipient");
            }
            catch (ServiceNotStartedException)
            {
                Console.WriteLine("Cardspace service not started");
            }
            catch (Exception e)
            {
                Console.WriteLine("Other exceptions :" + e.Message);
            }
            finally
            {
                Console.WriteLine("Press any key to exit.");
                Console.ReadKey(true);
            }
        }
    }
}
