using System;
using System.ServiceModel;


namespace Microsoft.ServiceModel.Samples.SecurityTokenService
{
    [Serializable]
    public abstract class WSTrustFaultException : FaultException
    {
        public static WSTrustFaultException FromException( Exception ex )
        {
            WSTrustFaultException faultException = ex as WSTrustFaultException;

            if ( faultException == null )
            {
                faultException = new RequestFailedFaultException( ex );
            }

            return faultException;
        }

        protected WSTrustFaultException( FaultReason reason, FaultCode code )
            : base( reason, code )
        {
        }
    }

    [Serializable]
    public class InvalidRequestFaultException : WSTrustFaultException
    {
        public InvalidRequestFaultException()
            : base( new FaultReason( "The request was invalid or malformed" ), new FaultCode( "sender", "http://www.w3.org/2003/05/soap-envelope", new FaultCode( "InvalidRequest", Constants.WSTrust.NamespaceUri.Uri ) ) )
        {
        }

        public InvalidRequestFaultException( Exception innerException )
            : base( new FaultReason( "The request was invalid or malformed" ), new FaultCode( "sender", "http://www.w3.org/2003/05/soap-envelope", new FaultCode( "InvalidRequest", Constants.WSTrust.NamespaceUri.Uri ) ) )
        {
        }
    }

    [Serializable]
    public class RequestFailedFaultException : WSTrustFaultException
    {
        public RequestFailedFaultException()
            : base( new FaultReason( "The specified request failed" ), new FaultCode( "sender", "http://www.w3.org/2003/05/soap-envelope", new FaultCode( "RequestFailed", Constants.WSTrust.NamespaceUri.Uri ) ) )
        {
        }

        public RequestFailedFaultException( Exception innerException )
            : base( new FaultReason( "The specified request failed" ), new FaultCode( "sender", "http://www.w3.org/2003/05/soap-envelope", new FaultCode( "RequestFailed", Constants.WSTrust.NamespaceUri.Uri ) ) )
        {
        }
    }
}
