using System;
using System.Collections.Generic;
using System.Configuration;
using System.IdentityModel.Claims;
using System.IdentityModel.Policy;
using System.IO;
using System.ServiceModel;

namespace Service
{

    public class AccessChecker : ServiceAuthorizationManager
    {
        private Dictionary<string, Claim[]> accessRequirements = null;

        public AccessChecker()
        {
            this.accessRequirements = new Dictionary<string, Claim[]>();

            OperationRequirementsConfigurationSection
                operationRequirementsConfigurationSection
                = ConfigurationManager.GetSection("operationRequirements")
                as OperationRequirementsConfigurationSection;

            OperationRequirementsCollection requirements =
                operationRequirementsConfigurationSection.OperationRequirements;
            List<Claim> roleClaims = null;
            foreach (OperationElement operationElement in requirements)
            {
                roleClaims = new List<Claim>(operationElement.Roles.Count);
                foreach (RoleElement roleElement in operationElement.Roles)
                {
                    roleClaims.Add(
                        new DefaultClaimSet(
                                ClaimSet.System, 
                                new Claim[] { 
                                    new Claim(
                                        "http://schemas.woodgrove.com/2005/05/ClaimType//Role", 
                                        roleElement.Name, 
                                        Rights.PossessProperty) })[0]);
                }

                this.accessRequirements.Add(
                    operationElement.Identifier,
                    roleClaims.ToArray());
            }
            
        }

        public override bool CheckAccess(OperationContext operationContext)
        {
            string header =
                operationContext.RequestContext.RequestMessage.Headers.Action;
            Claim[] requiredClaims = null;
            if (!(accessRequirements.TryGetValue(header, out requiredClaims)))
            {
                return false;
            }

            AuthorizationContext authorizationContext =
                operationContext.ServiceSecurityContext.AuthorizationContext;

            foreach (Claim requiredClaim in requiredClaims)
            {
                for (
                    int index = 0;
                    index < authorizationContext.ClaimSets.Count;
                    index++)
                {
                    if (
                        authorizationContext.ClaimSets[index].
                        ContainsClaim(requiredClaim))
                    {
                        return true;
                    }
                }
            }
            return false;
        }
    }
}