//-----------------------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//-----------------------------------------------------------------------------
using System;

using System.ServiceModel;
using System.ServiceModel.Channels;

namespace SecurityTokenService
{
    /// <summary>
    /// abstract base class containing properties that are shared by RST and RSTR classes
    /// </summary>
    public abstract class RequestSecurityTokenBase : BodyWriter
    {
        // private members
        protected string context;
        protected string tokenType;
        protected int keySize;
        protected EndpointAddress appliesTo;
        
        // Constructors
        /// <summary>
        /// Default constructor
        /// </summary>
        protected RequestSecurityTokenBase() : this(String.Empty,String.Empty,0, null)
        {
        }

        /// <summary>
        /// Parameterized constructor
        /// </summary>
        /// <param name="context">The value of the wst:RequestSecurityToken/@Context attribute in the request message, if any</param>
        /// <param name="tokenType">The content of the wst:RequestSecurityToken/wst:TokenType element in the request message, if any</param>
        /// <param name="keySize">The content of the wst:RequestSecurityToken/wst:KeySize element in the request message, if any</param>
        /// <param name="appliesTo">An EndpointRefernece corresponding to the content of the wst:RequestSecurityToken/wsp:AppliesTo element in the request message, if any</param>
        protected RequestSecurityTokenBase(string context, string tokenType, int keySize, EndpointAddress appliesTo )
            : base(true)
        {
            this.context = context;
            this.tokenType = tokenType;
            this.keySize = keySize;
            this.appliesTo = appliesTo;
        }

        // public properties

        /// <summary>
        /// Context for the RST/RSTR exchange. 
        /// The value of the wst:RequestSecurityToken/@Context attribute from RequestSecurityToken messages
        /// The value of the wst:RequestSecurityTokenResponse/@Context attribute from RequestSecurityTokenResponse messages        
        /// </summary>
        public string Context
        {
            get { return context; }
            set { context = value; }
        }

        /// <summary>
        /// The type of token requested or returned.
        /// The value of the wst:RequestSecurityToken/wst:TokenType element from RequestSecurityToken messages
        /// The value of the wst:RequestSecurityTokenResponse/wst:TokenType element from RequestSecurityTokenResponse messages       
        /// </summary>
        public string TokenType 
        { 
            get { return tokenType; }
            set { tokenType = value; }
        }


        /// <summary>
        /// The size of the requested proof key
        /// The value of the wst:RequestSecurityToken/wst:KeySize element from RequestSecurityToken messages
        /// The value of the wst:RequestSecurityTokenResponse/wst:KeySize element from RequestSecurityTokenResponse messages       
        /// </summary>
        public int KeySize
        {
            get { return keySize; }
            set { keySize = value; }
        }

        /// <summary>
        /// The EndpointAddress a token is being requested or returned for 
        /// The content of the wst:RequestSecurityToken/wsp:AppliesTo element from RequestSecurityToken messages
        /// The content of the wst:RequestSecurityTokenResponse/wsp:AppliesTo element from RequestSecurityTokenResponse messages       
        /// </summary>public int KeySize
        public EndpointAddress AppliesTo
        {
            get { return appliesTo; }
            set { appliesTo = value; }
        }
    }
}
