using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Configuration;
using System.IdentityModel;
using System.IdentityModel.Claims;
using System.IdentityModel.Policy;
using System.Runtime.Serialization;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;
using System.Text;
using System.Threading;
using System.Web.Security;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Hosting;
using System.Xml;

namespace SecurityTokenService
{
	public class AuthorizationPolicy : IAuthorizationPolicy
	{
		public AuthorizationPolicy()
		{
		}


		public ClaimSet Issuer
		{
			get
			{
                return DefaultClaimSet.System.Issuer;
			}
		}

		#region IAuthorizationPolicy Members

        bool IAuthorizationPolicy.Evaluate(
           EvaluationContext evaluationContext,
           ref object state)
        {
            List<Claim> claimsToAdd = new List<Claim>();
            ReadOnlyCollection<ClaimSet> inputClaims =
                evaluationContext.ClaimSets;
            for (int index = 0; index < inputClaims.Count; index++)
            {
                foreach (Claim claim in inputClaims[index].FindClaims(ClaimTypes.Name, null))
                {
                    string[] roles = Roles.Provider.GetRolesForUser(
                        (string)claim.Resource);
                    foreach (string role in roles)
                    {
                        claimsToAdd.Add(
                            new DefaultClaimSet(
                                ClaimSet.System, 
                                new Claim[] { 
                                    new Claim(
                                        "http://schemas.fabrikam.com/2005/05/ClaimType/Role", 
                                        role, 
                                        Rights.PossessProperty) })[0]);
                    }
                }
            }

            if (claimsToAdd.Count > 0)
            {
                evaluationContext.AddClaimSet(
                    this, new DefaultClaimSet(
                    this.Issuer, claimsToAdd));
            }

            return true;
        }

        ClaimSet IAuthorizationPolicy.Issuer
        {
            get
            {
                return this.Issuer;
            }
        }


		#endregion

		

        #region IAuthorizationComponent Members

        string IAuthorizationComponent.Id
        {
            get 
            {
                return Guid.NewGuid().ToString();
            }
        }

        #endregion
    }


}