using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    class MyCustomReplyChannel: IReplyChannel
    {
        private IReplyChannel innerChannel = null;

        public MyCustomReplyChannel(IReplyChannel innerChannel)
        {
            Console.WriteLine("Constructing reply channel.");
            this.innerChannel = innerChannel;
            this.innerChannel.Closed += new EventHandler(innerChannel_Closed);
        }

        protected void innerChannel_Closed(object sender, EventArgs e)
        {
            Console.WriteLine("Inner reply channel closed.");
            if (this.Closed != null)
            {
                this.Closed(this, e);
            }
        }


        #region IReplyChannel Members

        public IAsyncResult BeginReceiveRequest(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginReceiveRequest(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginTryReceiveRequest(TimeSpan timeout, AsyncCallback callback, object state)
        {
            Console.WriteLine("Reply channel BeginTryReceiveRequest.");
            return this.innerChannel.BeginTryReceiveRequest(timeout, callback, state);
        }

        public IAsyncResult BeginWaitForRequest(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public RequestContext EndReceiveRequest(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool EndTryReceiveRequest(IAsyncResult result, out RequestContext context)
        {
            Console.WriteLine("Reply channel EndTryReceiveRequest.");
            bool outcome = this.innerChannel.EndTryReceiveRequest(result, out context);
            Console.WriteLine("This is where I can apply my protocol to the inbound message.");
            return outcome;
        }

        public bool EndWaitForRequest(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public EndpointAddress LocalAddress
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public RequestContext ReceiveRequest(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public RequestContext ReceiveRequest()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool TryReceiveRequest(TimeSpan timeout, out RequestContext context)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool WaitForRequest(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region IChannel Members

        public T GetProperty<T>() where T : class
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region ICommunicationObject Members

        public void Abort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginClose(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginClose(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginOpen(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Close(TimeSpan timeout)
        {
            Console.WriteLine("Reply channel Close.");
            this.innerChannel.Close(timeout);
        }

        public void Close()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public event EventHandler Closed;

        public event EventHandler Closing;

        public void EndClose(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void EndOpen(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public event EventHandler Faulted;

        public void Open(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Open()
        {
            Console.WriteLine("Reply channel Open");
            this.innerChannel.Open();
        }

        public event EventHandler Opened;

        public event EventHandler Opening;

        public CommunicationState State
        {
            get 
            {
                Console.WriteLine("Retrieving reply channel state.");
                return this.innerChannel.State;
            }
        }

        #endregion
    }
}
