using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    class MyCustomRequestChannel: IRequestChannel
    {
        private IRequestChannel innerChannel = null;

        public MyCustomRequestChannel(IRequestChannel innerChannel)
        {
            Console.WriteLine("Constructing request channel.");
            this.innerChannel = innerChannel;
            this.innerChannel.Closed += new EventHandler(innerChannel_Closed);
        }

        protected void innerChannel_Closed(object sender, EventArgs e)
        {
            Console.WriteLine("Inner request channel closed.");
            if (this.Closed != null)
            {
                this.Closed(this, e);
            }
        }

        #region IRequestChannel Members

        public IAsyncResult BeginRequest(Message message, TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginRequest(Message message, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public Message EndRequest(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public EndpointAddress RemoteAddress
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public Message Request(Message message, TimeSpan timeout)
        {
            Console.WriteLine("Request channel request.");
            Console.WriteLine("This is where I can implement my protocol.");
            return this.innerChannel.Request(message,timeout);
        }

        public Message Request(Message message)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public Uri Via
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        #endregion

        #region IChannel Members

        public T GetProperty<T>() where T : class
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region ICommunicationObject Members

        public void Abort()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginClose(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginClose(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginOpen(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Close(TimeSpan timeout)
        {
            Console.WriteLine("Request channel Close");
            this.innerChannel.Close(timeout);
            
        }

        public void Close()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public event EventHandler Closed;

        public event EventHandler Closing;

        public void EndClose(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void EndOpen(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public event EventHandler Faulted;

        public void Open(TimeSpan timeout)
        {
            Console.WriteLine("Request channel Open");
            this.innerChannel.Open(timeout);
        }

        public void Open()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public event EventHandler Opened;

        public event EventHandler Opening;

        public CommunicationState State
        {
            get 
            {
                Console.WriteLine("Retrieving request channel state.");
                return this.innerChannel.State;    
            }
        }

        #endregion
    }
}
