using System;
using System.Collections.Generic;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    class MyCustomChannelFactory<TChannel>: ChannelFactoryBase<IRequestChannel>
    {
        IChannelFactory<TChannel> innerChannelFactory = null; 

        public MyCustomChannelFactory(MyCustomBindingElement bindingElement, BindingContext context)
            : base(context.Binding)
        {
            Console.WriteLine("Constructing the channel factory.");
            this.innerChannelFactory = context.BuildInnerChannelFactory<TChannel>();
            if (this.innerChannelFactory == null)
            {
                throw new InvalidOperationException("MyCustomChannelFactory requires an inner IChannelFactory.");
            }
        }

        protected override IRequestChannel OnCreateChannel(System.ServiceModel.EndpointAddress address, Uri via)
        {
            Console.WriteLine("Channel factory OnCreateChannel event.");
            IRequestChannel innerChannel = (IRequestChannel)this.innerChannelFactory.CreateChannel(address, via);
            return new MyCustomRequestChannel(innerChannel);
            
        }

        protected override IAsyncResult OnBeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnEndOpen(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        protected override void OnOpen(TimeSpan timeout)
        {
            Console.WriteLine("Channel factory OnOpen event.");
            this.innerChannelFactory.Open(timeout);
        }


    }
}
