using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
	class CustomChannelBase<TChannel> : ChannelBase	where TChannel : IChannel
	{
		ICustomChannelManager manager;
		TChannel innerChannel;

		protected CustomChannelBase(ChannelManagerBase manager, TChannel innerChannel): base(manager)
		{
			this.manager = (ICustomChannelManager)manager;
			this.innerChannel = innerChannel;
		}

		protected TChannel InnerChannel
		{
			get
			{
				return this.innerChannel;
			}
		}

		protected override void OnAbort()
		{
			this.innerChannel.Abort();
		}

		protected override IAsyncResult OnBeginClose(TimeSpan timeout, AsyncCallback callback, object state)
		{
			return this.innerChannel.BeginClose(timeout, callback, state);
		}

		protected override IAsyncResult OnBeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
		{
			return this.innerChannel.BeginOpen(timeout, callback, state);
		}

		protected override void OnClose(TimeSpan timeout)
		{
			this.innerChannel.Close(timeout);
		}

		protected override void OnEndClose(IAsyncResult result)
		{
			this.innerChannel.EndClose(result);
		}

		protected override void OnEndOpen(IAsyncResult result)
		{
			this.innerChannel.EndOpen(result);
		}

		protected override void OnOpen(TimeSpan timeout)
		{
			this.innerChannel.Open(timeout);
		}

		protected void OnReceive(Message message)
		{
			this.manager.OnReceive(message);
		}

		protected void OnSend(Message message)
		{
			this.manager.OnSend(message);
		}
	}
}
