using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    class MyCustomDuplexSessionChannel: IDuplexSessionChannel
    {
        private class PendingRead
        {
            public NetworkStream Stream = null;
            public byte[] Buffer = null;
        }

        private CommunicationState state = CommunicationState.Closed;
        private TcpClient tcpClient = null;
        private MessageEncoder encoder = null;
        private BufferManager bufferManager = null;
        private Dictionary<IAsyncResult, PendingRead> pendingReads = null;
        private TransportBindingElement bindingElement = null;
        private Uri uri = null;

        public MyCustomDuplexSessionChannel(Uri uri, TransportBindingElement bindingElement, MessageEncoderFactory encoderFactory, BufferManager bufferManager, TcpClient tcpClient)
        {
            this.uri = uri;
            this.bindingElement = bindingElement;
            this.tcpClient = tcpClient;
            this.state = CommunicationState.Created;
            this.bufferManager = bufferManager;

            this.encoder = encoderFactory.CreateSessionEncoder();

            this.pendingReads = new Dictionary<IAsyncResult, PendingRead>();
      }





        #region IInputChannel Members

        public IAsyncResult BeginReceive(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginReceive(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginTryReceive(TimeSpan timeout, AsyncCallback callback, object state)
        {
            if (this.state == CommunicationState.Opened)
            {
                byte[] buffer = this.bufferManager.TakeBuffer(this.tcpClient.Available);
                NetworkStream stream = this.tcpClient.GetStream();
                IAsyncResult result = stream.BeginRead(buffer, 0, buffer.Length, callback, state);
                PendingRead pendingRead = new PendingRead();
                pendingRead.Stream = stream;
                pendingRead.Buffer = buffer;
                this.pendingReads.Add(result, pendingRead);
                return result;
            }
            else
            {
                return null;
            }
            
        }

        public IAsyncResult BeginWaitForMessage(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public Message EndReceive(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool EndTryReceive(IAsyncResult result, out Message message)
        {
            message = null;

            PendingRead pendingRead = null;
            if (!(this.pendingReads.TryGetValue(result, out pendingRead)))
            {
                return false;
            }
            
            this.pendingReads.Remove(result);

            int size = 0;
            try
            {
                size = pendingRead.Stream.EndRead(result);
            }
            catch (IOException)
            {
                pendingRead.Stream.Close();
                
                this.Close(TimeSpan.Zero);
                
                
                return false;
            }

            byte[] data = this.bufferManager.TakeBuffer(size);
            Array.Copy(pendingRead.Buffer, data, size);
            this.bufferManager.ReturnBuffer(pendingRead.Buffer);

            message = encoder.ReadMessage(new ArraySegment<byte>(data, 0, size), this.bufferManager);
            message.Headers.To = this.uri;

            return true; ;
        }

        public bool EndWaitForMessage(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public EndpointAddress LocalAddress
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public Message Receive(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public Message Receive()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool TryReceive(TimeSpan timeout, out Message message)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public bool WaitForMessage(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region IChannel Members

        public T GetProperty<T>() where T : class
        {
            if(typeof(T) == typeof(FaultConverter))
            {
                return new MyFaultConverter() as T;
            }
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion

        #region ICommunicationObject Members

        public void Abort()
        {
            this.Close(TimeSpan.Zero);
        }

        public IAsyncResult BeginClose(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginClose(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginOpen(TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginOpen(AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Close(TimeSpan timeout)
        {
            if (this.state != CommunicationState.Closed)
            {
                this.tcpClient.Close();

                this.state = CommunicationState.Closed;

                this.pendingReads.Clear();

                if (this.Closed != null)
                {
                    this.Closed(this, null);
                }
            }
        }

        public void Close()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public event EventHandler Closed;

        public event EventHandler Closing;

        public void EndClose(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void EndOpen(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public event EventHandler Faulted;

        public void Open(TimeSpan timeout)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void Open()
        {
            this.state = CommunicationState.Opened;
            if (this.Opened != null)
            {
                this.Opened(this, null);
            }
        }

        public event EventHandler Opened;

        public event EventHandler Opening;

        public CommunicationState State
        {
            get 
            {
                return this.state;
            }
        }

        #endregion

        #region IOutputChannel Members

        public IAsyncResult BeginSend(Message message, TimeSpan timeout, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public IAsyncResult BeginSend(Message message, AsyncCallback callback, object state)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public void EndSend(IAsyncResult result)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public EndpointAddress RemoteAddress
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        public void Send(Message message, TimeSpan timeout)
        {
            ArraySegment<byte> bytes = this.encoder.WriteMessage(message, (int)this.bindingElement.MaxReceivedMessageSize, this.bufferManager);
            NetworkStream stream = this.tcpClient.GetStream();
            stream.Write(bytes.Array, 0, bytes.Count);
            bufferManager.ReturnBuffer(bytes.Array);
        }

        public void Send(Message message)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public Uri Via
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        #endregion

        #region ISessionChannel<IDuplexSession> Members

        public IDuplexSession Session
        {
            get { throw new Exception("The method or operation is not implemented."); }
        }

        #endregion
    }
}
