using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;

namespace Extensibility
{
    public class MyCustomTransportBindingElement: TransportBindingElement
    {
        public MyCustomTransportBindingElement()
        {
            Console.WriteLine("Constructing binding element.");
           
        }

        public MyCustomTransportBindingElement(MyCustomTransportBindingElement original)
        {
            Console.WriteLine("Copying binding element.");
        }

        public override BindingElement Clone()
        {
            Console.WriteLine("Cloning binding element.");
            return new MyCustomTransportBindingElement(this);
        }

        public override T GetProperty<T>(BindingContext context)
        {
            Console.WriteLine("Querying property {0}", typeof(T));
            return context.GetInnerProperty<T>();
        }

        public override bool CanBuildChannelFactory<TChannel>(BindingContext context)
        {
            Console.WriteLine("Querying if the binding element can build a channel factory of type {0}.", typeof(TChannel).Name);
            if (typeof(TChannel) == typeof(IDuplexSessionChannel))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public override bool CanBuildChannelListener<TChannel>(BindingContext context)
        {
            Console.WriteLine("Querying if the binding element can build a listener of type {0}.", typeof(TChannel).Name);
            if (typeof(TChannel) == typeof(IDuplexSessionChannel))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            Console.WriteLine("Asking the binding element for a channel factory of type {0}.", typeof(TChannel).Name);
            return (IChannelFactory<TChannel>)(object)new MyCustomChannelFactory<TChannel>(this,context);
        }

        public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
        {
            Console.WriteLine("Asking the binding element for a listener of type {0}.", typeof(TChannel).Name);
            return(IChannelListener<TChannel>)(object)new MyCustomChannelListener<TChannel>(this,context);
        }

        public override string Scheme
        {
            get 
            {
                return "any.tcp";
            }
        }
    }
}
