using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;

namespace PublicationSubscription
{
	public partial class NotificationStreamReader: INotificationStreamReader
	{
		private Stream source = null;
		
		public event ReadValueHandler NextValue;
		public event EndOfStreamHandler EndOfStream;

		private NotificationStreamReader()
		{
		}

		public NotificationStreamReader(Stream source)
		{
			this.source = source;
		}
		
		private void Read(object state)
		{
			ISubscriptionStreamControlSignal controlSignal = null;
			byte controlSignalSize = 0;
			int noiseBytes = 0;
			byte[] controlSignalBuffer = null;
			byte[] buffer = null;

			buffer = new byte[1];
			int bytesRead = this.source.Read(buffer, 0, 1);
			if (bytesRead > 0)
			{
				controlSignalSize = buffer[0];

				while (true)
				{
					buffer = new byte[controlSignalSize];
					bytesRead = this.source.Read(buffer, 0, controlSignalSize);
									
					noiseBytes = 0;
					foreach (byte currentByte in buffer)
					{
						if (currentByte == 0)
						{
							noiseBytes++;
						}
						else
						{
							break;
						}
					}
					if (noiseBytes > 0)
					{
						if (noiseBytes == controlSignalSize)
						{
							Console.WriteLine("Bytes read: {0}", bytesRead.ToString());
							buffer = new byte[1024];
							while(this.source.Read(buffer,0,1024)>0)
							{
								Console.WriteLine("Completing read.");
							}
							break;
						}
						controlSignalBuffer = new byte[controlSignalSize];
						Array.Copy(buffer, noiseBytes, controlSignalBuffer, 0, buffer.Length - noiseBytes);
						bytesRead = this.source.Read(controlSignalBuffer, controlSignalBuffer.Length - noiseBytes, noiseBytes);
						if (bytesRead < noiseBytes)
						{
							throw new IOException();
						}
					}
					else
					{
						controlSignalBuffer = buffer;
					}


					controlSignal = new SubscriptionStreamControlSignal(controlSignalBuffer);

					buffer = new byte[controlSignal.DataPointValueSize];
					bytesRead = this.source.Read(buffer, 0, buffer.Length);
					if (bytesRead < buffer.Length)
					{
						//throw new IOException();
					}

					if (this.NextValue != null)
					{
						this.NextValue(this, controlSignal.DataPointIdentifier.ToString(), BitConverter.ToInt32(buffer, 0).ToString());
					}
				}
			}
			
			if (this.EndOfStream != null)
			{
				this.EndOfStream(this);
				this.source.Close();
			}
		}

		#region INotificationStreamReader Members

		Stream INotificationStreamReader.Source
		{
			get 
			{
				return this.source;
			}
		}

		void INotificationStreamReader.Start()
		{
			//ThreadPool.QueueUserWorkItem(new WaitCallback(this.Read));
			this.Read(null);
		}

		#endregion

		

		
}
}
