using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Client
{
	delegate void RetrievePictureDelegate(object state);

	[ServiceContract]
	public interface IPictureServer
	{
		[OperationContract]
		Stream GetPicture(string pictureName);
	}

    public class PictureClient : ClientBase<IPictureServer>, IPictureServer
    {
        public PictureClient(string endpointConfigurationName)
            : base(endpointConfigurationName)
        {
        }

        #region IPictureServer Members

        public Stream GetPicture(string pictureName)
        {
            return base.Channel.GetPicture(pictureName);
        }

        #endregion
    }


	public partial class Main : Form
	{
		public Main()
		{
			InitializeComponent();
		}

		private void Main_Load(object sender, EventArgs e)
		{
			
		}

		private void RetrievePicture(object state)
		{
			if (this.InvokeRequired)
			{
				PictureClient pictureClient = new PictureClient("PictureServer");
                Stream pictureStream = pictureClient.GetPicture(ConfigurationManager.AppSettings["PictureName"]);
                pictureClient.Close();
				
				this.Invoke(new RetrievePictureDelegate(this.RetrievePicture),new object[]{pictureStream});
			}
			else
			{
				Bitmap bitMap = new Bitmap((Stream)state);
				this.Picture.Image = bitMap;
			}
		}

		private void StartButton_Click(object sender, EventArgs e)
		{
			ThreadPool.QueueUserWorkItem(new WaitCallback(this.RetrievePicture));
			
			
		}
	}
}