using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace PublicationSubscription
{
	public class SubscriptionManager: ISubscriptionManager
	{
		private string subscriber = null;
		private IDataPoint[] subscriptions = null;
		private object subscriptionsLock = null;
		private object stateLock = null;
		private NextValueHandler nextValueHandler = null;
		private SubscriptionManagerState state = SubscriptionManagerState.Inactive;
		
		private SubscriptionManager()
		{
			this.subscriptions = new IDataPoint[] { };
			this.subscriptionsLock = new object();
			this.stateLock = new object();
		}

		public SubscriptionManager(string subscriber, NextValueHandler nextValueHandler):this()
		{
			this.subscriber = subscriber;
			this.nextValueHandler = nextValueHandler;
		}

		#region IDataSubscriptionManager Members

		string ISubscriptionManager.Subscriber
		{
			get 
			{ 
				return this.subscriber;
			}
		}

		IDataPoint[] ISubscriptionManager.Subscriptions
		{
			get 
			{
				lock (this.subscriptionsLock)
				{
					return this.subscriptions;
				}
			}
		}

		SubscriptionManagerState ISubscriptionManager.State
		{
			get 
			{
				lock (this.stateLock)
				{
					return this.state;
				}
			}
		}

		void ISubscriptionManager.AddSubscription(IDataPoint dataPoint)
		{
			TimeSpan? subscriptionTimeout = null;

			lock (this.subscriptionsLock)
			{
				foreach (IDataPoint currentSubscription in this.subscriptions)
				{
					if (currentSubscription.GetType() == dataPoint.GetType())
					{
						return;
					}
				}

				dataPoint.NextValue += this.nextValueHandler;

				List<IDataPoint> currentSubscriptions = new List<IDataPoint>(this.subscriptions);
				currentSubscriptions.Add(dataPoint);
				this.subscriptions = currentSubscriptions.ToArray();
			}

			lock (this.stateLock)
			{
				this.state = SubscriptionManagerState.Active;
			}

		}

		#endregion

		#region IDisposable Members

		void IDisposable.Dispose()
		{
			lock (this.stateLock)
			{
				this.state = SubscriptionManagerState.Inactive;
			}

			lock (this.subscriptionsLock)
			{
				foreach (IDataPoint subscription in this.subscriptions)
				{
					subscription.NextValue -= this.nextValueHandler;
				}

			}
		}

		#endregion

		
}
}
