using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.ServiceModel;
using System.Text;
using System.Threading;

namespace PublicationSubscription
{
	public class PublisherService: IPublisher
	{
		private RandomDataPoint randomDataPoint = null;
		private Dictionary<string, string> knownSubscribers = null;

		public PublisherService()
		{
			this.randomDataPoint = new RandomDataPoint("Random", "Random data point", int.Parse(ConfigurationManager.AppSettings["RandomDataPointFrequencyInMilliseconds"]),false);
			
			this.knownSubscribers = new Dictionary<string, string>();
		}

		

		private void ValidateSubscriber(string subscriberIdentity, out bool valid, out string configuration)
		{
            valid = true;
            configuration = "Subscriber";
		}

		private void ValidateDataPoint(string dataPointIdentifier, out bool valid)
		{
			valid = false;
			if (dataPointIdentifier == ((IDataPoint)this.randomDataPoint).Identifier.ToString())
			{
				valid = true;
			}
		}

		
		
		

		#region IPublisher Members

		KnownDataPoint[] IPublisher.GetKnownDataPoints()
		{
			return new KnownDataPoint[] { new KnownDataPoint(this.randomDataPoint) };
		}

		void IPublisher.Subscribe(KnownDataPoint[] dataPoints, out bool subscriptionAccepted)
		{
			subscriptionAccepted = false;
			string dataPointIdentifier = null;
			if (dataPoints.Length == 1)
			{
				dataPointIdentifier = dataPoints[0].Identifier;
				this.ValidateDataPoint(dataPointIdentifier, out subscriptionAccepted);
			}

			string configuration = null; 
			if(subscriptionAccepted)
			{
				this.ValidateSubscriber(OperationContext.Current.ServiceSecurityContext.WindowsIdentity.Name, out subscriptionAccepted, out configuration);
			}

			if (subscriptionAccepted)
			{

				ThreadPool.QueueUserWorkItem(new WaitCallback(((IPublishingAgent)new PublishingAgent(configuration, dataPointIdentifier)).Activate) , null);
			}
		}

		#endregion
}
}
