using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Windows.Forms;

using WindowsCommunicationFoundationHandsOn.School;

namespace WindowsCommunicationFoundationHandsOn
{
	

	public partial class MainForm : Form
	{
		private const string CorrectImage = @"Check.gif";
		private const string IncorrectImage = @"Cross.gif";
		
		private delegate void AddQuizItemDelegate(QuizItem item);
		private delegate void DisplayGradeDelegate(QuizResponse response);
		private delegate void DisplayAccouncementDelegate(string announcement);

		private Image correctImage = null;
		private Image incorrectImage = null;
		private int currentItemIndex = -1;
		private QuizChild quizClient = null;
		private List<QuizItem> quizItems = null;
		
		
		public MainForm(string[] parameters)
		{
			InitializeComponent();
			
			this.quizItems = new List<QuizItem>();
			this.quizClient = new QuizChild(this, parameters);
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			this.correctImage = new Bitmap(MainForm.CorrectImage);
			this.incorrectImage = new Bitmap(MainForm.IncorrectImage);

			this.EnableNavigationButtons();
			this.EnableRadioButtons();
		}

		public void AddQuizItem(QuizItem item)
		{
			if (this.InvokeRequired)
			{
				AddQuizItemDelegate addQuizItemDelegate = new AddQuizItemDelegate(this.AddQuizItem);
				this.Invoke(addQuizItemDelegate, new object[] { item });
			}
			else
			{
				this.quizItems.Add(item);
				if (this.quizItems.Count == 1)
				{
					this.currentItemIndex = 0;
					this.DisplayItem();
				}
				this.EnableNavigationButtons();
			}
		}

		public void DisplayAnnouncement(string announcement)
		{
			if (this.InvokeRequired)
			{
				DisplayAccouncementDelegate displayAccouncementDelegate = new DisplayAccouncementDelegate(this.DisplayAnnouncement);
				this.Invoke(displayAccouncementDelegate, new object[] { announcement });
			}
			else
			{
				this.Message.Text = announcement;
				
			}
		}

		public void DisplayGrade(QuizResponse response)
		{
			if (this.InvokeRequired)
			{
				DisplayGradeDelegate displayGradeDelegate = new DisplayGradeDelegate(this.DisplayGrade);
				this.Invoke(displayGradeDelegate, new object[] { response });
			}
			else
			{
				QuizItem item;
				int limit = this.quizItems.Count;
				for(int index = 0;index < limit;index++)
				{
					item = this.quizItems[index];
					if (string.Compare(item.Identifier, response.Item.Identifier, true) == 0)
					{
						int responseLimit = item.Responses.Length;
						QuizResponse currentResponse;
						for(int responseIndex = 0;responseIndex < responseLimit;responseIndex++)
						{
							currentResponse = item.Responses[responseIndex];
							if(currentResponse.Submitted)
							{
								currentResponse.Correct = response.Correct;
								this.quizItems[index].Responses[responseIndex] = currentResponse;
								if (index == this.currentItemIndex)
								{
									this.DisplayGrade();
								}
								break;
							}
						}
						break;
						
					}
				}

			}
		}

		private void DisplayGrade()
		{
			QuizItem item = this.quizItems[this.currentItemIndex];
			if (item.Correct != null)
			{
				this.ResultImage.SizeMode = PictureBoxSizeMode.StretchImage;
				this.ResultImage.Image = item.Correct.Value ? this.correctImage : this.incorrectImage;
			}
			else
			{
				this.ResultImage.Image = null;
			}
			this.ResultImage.Invalidate();
		}

		private void DisplayItem()
		{
			QuizItem item = this.quizItems[this.currentItemIndex];
			this.QuizPicture.SizeMode = PictureBoxSizeMode.StretchImage;
			MemoryStream stream = new MemoryStream(item.ItemImage);
			Bitmap bitmap = new Bitmap(stream);
			this.QuizPicture.Image = bitmap;
			
			this.QuizQuestion.Text = item.Text;
			this.QuizRadio1.Text = item.Responses[0].ResponseText;
			this.QuizRadio2.Text = item.Responses[1].ResponseText;
			this.QuizRadio3.Text = item.Responses[2].ResponseText;
			this.QuizRadio4.Text = item.Responses[3].ResponseText;

			RadioButton[] options = new RadioButton[] { this.QuizRadio1, this.QuizRadio2, this.QuizRadio3, this.QuizRadio4 };
			foreach (RadioButton option in options)
			{
				if(item.Submitted)
				{
					if (string.Compare(option.Text, item.ResponseText, true) == 0)
					{
						option.Checked = true;
					}
				}
				else
				{
					option.Checked = false;
				}

			}

			this.DisplayGrade();
			
			
			this.EnableResponseButton();
			this.EnableRadioButtons();
			

			
		}

		private void EnableNavigationButtons()
		{
			this.QuizForwardButton.Enabled = (this.currentItemIndex < this.quizItems.Count - 1);
			this.QuizBackButton.Enabled = (this.currentItemIndex > 0);
		}

		private void QuizResponse_CheckedChanged(object sender, EventArgs e)
		{
			RadioButton option = (RadioButton)sender;
			if (option.Checked)
			{
				QuizResponse[] responses = this.quizItems[this.currentItemIndex].Responses;
				QuizResponse currentResponse;
				int limit = responses.Length;
				for (int index = 0; index < limit; index++)
				{
					currentResponse = responses[index];
					if (string.Compare(currentResponse.ResponseText, option.Text, true) == 0)
					{
						currentResponse.Submitted = true;
						this.quizItems[this.currentItemIndex].Responses[index] = currentResponse;
						break;
					}
				}
				this.EnableResponseButton();
			}
		}

		private void EnableResponseButton()
		{
			QuizItem item = this.quizItems[this.currentItemIndex];
			bool enabled = (!(item.Submitted));
			if (enabled)
			{
				this.OKButton.Enabled = ((this.QuizRadio1.Checked) || (this.QuizRadio2.Checked) || (this.QuizRadio3.Checked) || (this.QuizRadio4.Checked));
			}
			else
			{
				this.OKButton.Enabled = false;
			}
			
		}

		private void EnableRadioButtons()
		{
			bool enabled;
			if (this.currentItemIndex >= 0)
			{
				QuizItem item = this.quizItems[this.currentItemIndex];
				enabled = (!(item.Submitted));
			}
			else
			{
				enabled = false;
			}
			RadioButton[] options = new RadioButton[] { this.QuizRadio1, this.QuizRadio2, this.QuizRadio3, this.QuizRadio4 };
			foreach (RadioButton option in options)
			{
				option.Enabled = enabled;
			}
		}

		private void OKButton_Click(object sender, EventArgs e)
		{
			QuizItem currentQuizItem = this.quizItems[this.currentItemIndex];
			currentQuizItem.Submitted = true;

			QuizResponse response = new QuizResponse();
			response.Item = new QuizItem();
			response.Item.Identifier = currentQuizItem.Identifier;

			foreach (QuizResponse responseOption in currentQuizItem.Responses)
			{
				if (responseOption.Submitted)
				{
					response.ResponseText = responseOption.ResponseText;
					break;
				}
			}
			
			this.quizClient.SendResponse(response);

			this.quizItems[this.currentItemIndex] = currentQuizItem;

			this.EnableResponseButton();
		}

		private void QuizForwardButton_Click(object sender, EventArgs e)
		{
			this.currentItemIndex++;
			this.NavigationButton_Click();
		}

		private void QuizBackButton_Click(object sender, EventArgs e)
		{
			this.currentItemIndex--;
			this.NavigationButton_Click();
		}

		private void NavigationButton_Click()
		{
			this.EnableNavigationButtons();
			this.DisplayItem();
		
		}

		
		
	}
}