using System;
using System.Collections.Generic;
using System.Configuration;
using System.Drawing;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Security;
using System.ServiceModel.Security.Tokens;
using System.Text;

namespace WindowsCommunicationFoundationHandsOn.School
{
	


	public class QuizTeacher : IQuizQuestion, IQuizResponse, IQuizManagement, IDisposable
	{
		protected const string MeshIdentifierKey = @"MeshIdentifier";
		protected const string QuizQuestionEndpointKey = @"QuizQuestionEndpoint";
		protected const string QuizManagementEndpointKey = @"QuizManagementEndpoint";
		protected const string QuizResponseEndpointKey = @"QuizResponseEndpoint";
		protected const string PeerChannelAddressPrefix = @"net.p2p://";
		protected const string CertificateSubjecDistinguishedName = @"CN=FabrikamEnterprises";
        protected const string PeerBindingKey = @"PeerBinding";
        
		protected ServiceHost resolverService = null;
        protected QuizQuestionClient questionClient = null;
        protected QuizManagementClient managementClient = null;
        
        protected string meshIdentifier = null;
		protected InstanceContext site = null;
        protected Binding binding = null;
        
		
		MainForm form = null;

        Dictionary<string, QuizResponseClient> responseClients = new Dictionary<string, QuizResponseClient>();

        public QuizTeacher(MainForm form)
        {
            this.form = form;

            this.site = new InstanceContext(this);

            this.managementClient = new QuizManagementClient(this.site, QuizTeacher.QuizManagementEndpointKey);
            this.managementClient.Open();

            this.questionClient = new QuizQuestionClient(this.site, QuizTeacher.QuizQuestionEndpointKey);
            this.questionClient.Open();
        }

        
        public void SendAccouncement(string announcement)
		{
			managementClient.Announce(announcement);
		}

		public void SendItems(Quiz quiz)
		{
			foreach (QuizItem item in quiz.Items)
			{
				questionClient.AddItem(item);
			}
		}

		public void SendGrade(QuizResponse response)
		{
            QuizResponseClient responseClient = null;
            if (this.responseClients.TryGetValue(response.Responder.Name, out responseClient))
            {
                responseClient.SendResponse(response);
            }
			
		}

        #region IQuizQuestion Members

		void IQuizQuestion.AddItem(QuizItem item)
		{
		}

		#endregion

		#region IQuizResponse Members

		void IQuizResponse.SendResponse(QuizResponse response)
		{
			this.form.AddResponse(response);
		}

		#endregion

		
		#region IDisposable Members

		void IDisposable.Dispose()
		{
            this.questionClient.Close();
            this.managementClient.Close();
			
            foreach (QuizResponseClient responseClient in this.responseClients.Values)
            {
                responseClient.Close();
            }
        }

		#endregion

		#region IQuizManagement Members

        void IQuizManagement.Join(Student student)
        {
            if (!(this.responseClients.ContainsKey(student.Name)))
            {
                if(this.meshIdentifier == null)
                {
                    this.meshIdentifier = ConfigurationManager.AppSettings[QuizTeacher.MeshIdentifierKey];
                }
                string endpoint =
                    ConfigurationManager.
                    AppSettings[QuizTeacher.QuizResponseEndpointKey];
                EndpointAddress address =
                    new EndpointAddress(
                        string.Format("{0}{1}{2}/{3}",
                            QuizTeacher.PeerChannelAddressPrefix,
                            this.meshIdentifier,
                            student.Name,
                            endpoint));

                if(this.binding == null)
                {
                    binding = new NetPeerTcpBinding(QuizTeacher.PeerBindingKey);
                }

                QuizResponseClient responseClient = new QuizResponseClient(this.site, this.binding, address);
                responseClient.ClientCredentials.Peer.SetCertificate(
                    StoreLocation.LocalMachine,
                    StoreName.My,
                    X509FindType.FindBySubjectDistinguishedName,
                    QuizTeacher.CertificateSubjecDistinguishedName);
                responseClient.ClientCredentials.Peer.PeerAuthentication.CertificateValidationMode = X509CertificateValidationMode.PeerTrust;
                responseClient.ClientCredentials.Peer.PeerAuthentication.TrustedStoreLocation = StoreLocation.CurrentUser;
                responseClient.Open();

                this.responseClients.Add(student.Name, responseClient);

                this.form.UpdateStudents(student.Image);
            }
        }


		void IQuizManagement.Announce(string announcement)
		{
		}

		#endregion
    }

	
}
