using System;
using System.Collections.Generic;
using System.IO;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Xml;

namespace RESTPOXService
{
    [ServiceContract]
    public interface IReallySimpleSyndication
    {
        [OperationContract(Action = "*", ReplyAction = "*")]
        Message Resource(Message input);
    }

    [ServiceBehavior(
        InstanceContextMode = InstanceContextMode.Single,
        ConcurrencyMode=ConcurrencyMode.Multiple)]
    public class ReallySimpleSyndicationServer : IReallySimpleSyndication
    {
        
        Message IReallySimpleSyndication.Resource(Message input)
        {
            HttpRequestMessageProperty httpRequestProperty
                = (HttpRequestMessageProperty)
                input.Properties[HttpRequestMessageProperty.Name];
            string query = httpRequestProperty.QueryString;
            string fileName = string.Format("{0}.xml", query);

            switch (httpRequestProperty.Method)
            {
                case "GET":
                    Message message = null;
                    if (File.Exists(fileName))
                    {
                        XmlDocument document = new XmlDocument();
                        document.Load(fileName);

                        message = Message.CreateMessage(
                            MessageVersion.None, 
                            "*", 
                            new XmlNodeReader(document.DocumentElement));

                    }
                    else
                    {
                        message = Message.CreateMessage(
                            MessageVersion.None,
                            "*");
                    }

                    return message;

                default:
                    return Message.CreateMessage(
                        MessageVersion.None, "*");

            }

        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            using (ServiceHost host = new ServiceHost(typeof(ReallySimpleSyndicationServer)))
            {
                host.Open();
                Console.WriteLine("The service is ready.");

                Console.ReadKey(true);
                host.Close();
            }
        }
    }
}
