using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Text;
using System.Diagnostics;
using System.Management;
using System.Management.Instrumentation;
using System.Runtime.InteropServices;
using System.ServiceModel.Description;
using System.Threading;

namespace Fabrikam
{
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class TradeRecorder : ITradeRecorder
    {
        private const string CounterCategoryName = "TradeRecording";
        private const string VolumeCounterName = "Trade Volume";

        private PerformanceCounterCategory counterCategory = null;


        public TradeRecorder()
        {
            if (PerformanceCounterCategory.Exists(TradeRecorder.CounterCategoryName))
            {
                PerformanceCounterCategory.Delete(TradeRecorder.CounterCategoryName);
            }

            CounterCreationData volumeCounter = new CounterCreationData(TradeRecorder.VolumeCounterName, "Volume of trading.", PerformanceCounterType.NumberOfItemsHEX32);
            CounterCreationDataCollection counterCollection = new CounterCreationDataCollection(new CounterCreationData[] { volumeCounter });

            this.counterCategory = PerformanceCounterCategory.Create(TradeRecorder.CounterCategoryName, "Trade Recording Data", PerformanceCounterCategoryType.MultiInstance, counterCollection);

        }

        private PerformanceCounter volumeCounter = null;
        public void InitializeCounters(ServiceEndpointCollection endpoints)
        {

            while (true)
            {
                try
                {
                    string condition = "SELECT * FROM Service WHERE Name=\"TradeRecorder\"";
                    SelectQuery query = new SelectQuery(condition);
                    ManagementScope managementScope = new ManagementScope(@"\\.\root\ServiceModel", new ConnectionOptions());
                    ManagementObjectSearcher searcher = new ManagementObjectSearcher(managementScope, query);
                    ManagementObjectCollection instances = searcher.Get();
                    foreach (ManagementBaseObject instance in instances)
                    {
                        PropertyData data = instance.Properties["CounterInstanceName"];

                        this.volumeCounter = new PerformanceCounter(TradeRecorder.CounterCategoryName, TradeRecorder.VolumeCounterName, data.Value.ToString());
                        this.volumeCounter.ReadOnly = false;
                        this.volumeCounter.RawValue = 0;

                        break;
                    }

                    break;
                }
                catch (COMException)
                {
                    Thread.Sleep(500);
                }

            }


            if (this.volumeCounter != null)
            {
                Console.WriteLine("Volume counter initialized.");
            }
            Console.WriteLine("Counters initialized.");

        }

        #region ITradeRecorder Members
        private long tradeCount = 0;
        void ITradeRecorder.RecordTrades(Trade[] trades)
        {
            Console.WriteLine("Recording trade ...");
            lock (this)
            {
                while (this.volumeCounter == null)
                {
                    Thread.Sleep(100);
                }
            }



            foreach (Trade trade in trades)
            {
                this.tradeCount += ((trade.Count != null) ? trade.Count.Value : 0);

                this.volumeCounter.RawValue = this.tradeCount;

                Console.WriteLine(string.Format("Recorded trade for {0}", trade));
            }

        }

        #endregion
    }
}
